%{
/* Configuration file parser
 *
 * Copyright (c) 2002, Martin Hedenfalk <mhe@home.se>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* $Id: lexer.l,v 1.4 2002/04/10 12:29:48 mhe Exp $
 */

#include <string.h>
#include "libcfg.h"

#define YY_DECL int yylex YY_PROTO(( cfg_t *cfg ))

/* temporary buffer for the C-style string scanner
 * FIXME: doesn't check for buffer overflow!
 */
char *string_buf_ptr = 0;
char string_buf[1024];
%}

 /* start conditions
  */
%x comment
%x str

%%

[ \t]+    /* eat up whitespace */

[\n]	 cfg->line++; /* keep track of line number */

("#"|"//")[^\n]*     /* eat up one-line comments */

 /* special keywords/symbols
  */
"{"           return '{';
"}"           return '}';
"="           return '=';
","           return ',';
<<EOF>>       return EOF;

 /* handle multi-line C-style comments
  */
"/*"         BEGIN(comment);
<comment>[^*\n]*        /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   /* eat up '*'s not followed by '/'s */
<comment>\n             cfg->line++;
<comment>"*"+"/"        BEGIN(INITIAL);

 /* handle C-style strings
  */
"\""    string_buf_ptr = string_buf; BEGIN(str);
<str>\"  { /* saw closing quote - all done */
	BEGIN(INITIAL);
	*string_buf_ptr = '\0';
	yylval.s = strdup(string_buf);
	return CFGT_STR;
 }
<str>\n   cfg_error(cfg, "unterminated string constant");
<str>\\[0-7]{1,3} {
	/* octal escape sequence */
	int result;
	sscanf(yytext + 1, "%o", &result);
	if(result > 0xFF)
		cfg_error(cfg, "invalid octal number '%s'\n", yytext);
	*string_buf_ptr++ = result;
 }
<str>\\[0-9]+   {
	cfg_error(cfg, "bad escape sequence '%s'\n", yytext);
 }
<str>\\n  *string_buf_ptr++ = '\n';
<str>\\t  *string_buf_ptr++ = '\t';
<str>\\r  *string_buf_ptr++ = '\r';
<str>\\b  *string_buf_ptr++ = '\b';
<str>\\f  *string_buf_ptr++ = '\f';
<str>\\(.|\n)  *string_buf_ptr++ = yytext[1];
<str>[^\\\n\"]+  {
	char *yptr = yytext;
	while ( *yptr )
		*string_buf_ptr++ = *yptr++;
 }

 /* a signed integer
  */
-?[0-9]+        yylval.i = atoi(yytext); return CFGT_INT;

 /* an unquoted string
  */
[-[:alnum:]/\.]+  yylval.s = strdup(yytext); return CFGT_STR;

%%

int yywrap(void)
{
    return 1;
}

void dummy_function(void)
{
	/* please compiler :-)
	 * otherwise "defined but not used" warning
	 */
	yyunput(0, 0);
}
