/* Open.Net Authentication Daemon
 * Authentication of users and controlling the dynamic firewalls
 * Written by Martin Hedenfalk <mhe@it.kth.se>
 *
 * Copyright (c) 2002, Royal Institute of Technology, Stockholm, Sweden
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* $Id: auth.c,v 1.13 2002/04/06 21:27:11 mhe Exp $
 */

#include "common.h"
#include "xmalloc.h"

/* Authenticate user and open up a rule in firewall
 */
req_status_t login(userdata_t *ud, char *password)
{
	req_status_t status;

	if(!ud->domain)
		status = REQ_STATUS_NODOMAIN;
	else
		status = authenticate(ud->username, ud->domain->name, password);

	if(status == REQ_STATUS_OK) {
		/* check if the user is not already logged in
		 */
		status = monitor_check_user(ud);

		if(status == REQ_STATUS_OK) {
			/* try to open a rule for the user
			 */
			status = fw_open(ud);

			if(status == REQ_STATUS_OK) {
				logmsg(LOG_INFO, "login %s@%s ip: %s, mac: %s",
					   ud->username, ud->domain->name,
					   inet_ntoa(ud->ip), strmac(ud->mac));

				status = monitor_insert(ud);
			}
		}
	} else
		failed_auths++;

	return status;
}

/* Close firewall rule for the specified user
 */
req_status_t logout(userdata_t *ud)
{
	req_status_t status;

	set_mstatus(ud, MS_LOGOUT);
	logmsg(LOG_INFO, "logout %s@%s ip: %s, mac: %s",
		   ud->username, ud->domain->name, inet_ntoa(ud->ip), strmac(ud->mac));

	/* FIXME: check the order!
	 */

	status = fw_close(ud);

	if(status == REQ_STATUS_OK) {
		/* Ok, the firewall has closed the rule for the user, now
		 * remove him from the list. If the firewall failed above,
		 * the user will still be in the "logged in" list, and if
		 * the next probe is missed we'll try to close the rule
		 * again and again...
		 */
		status = monitor_remove(ud);
	}
	return status;
}

/* Reset all configured firewalls
 */
void reset_firewalls(void)
{
	int i;

	for(i = 0; i < num_domain; i++) {
		fw_reset(&domains[i]);
		/* don't care about return status, we'll continue anyway
		 * (there's nothing we can do, right?)
		 */
	}
}
