#ifndef _llhash_h_
#define _llhash_h_

#include "common.h"

struct traffic {
	int key;    /* hash key */
	int packets; /* number of packets received during last TIMEOUT seconds */
	int misses;
	int failed_logouts;
	struct in_addr ip;
	unsigned char mac[6]; /* MAC address */
};
int makekey(unsigned char mac[6]);

#define LLHITEM struct traffic *
#define LLHSLOTS 4096
/*#define LLHKEY(p) makekey((p)->mac)*/
#define LLHKEY(p) ((struct traffic *)p)->key

/*
 *  Hash table item structure
 */
struct item
{
        LLHITEM d;
        struct item *next;
};

/*
 *  The hash table itself
 */
typedef struct item *LLHTABLE[LLHSLOTS];

/*
 *  Hash function
 *
 *  Parameters:
 *
 *    key         Some hash table item's key
 *
 *  Return values:
 *
 *    >= 0        The item's hash value
 *
 *    <= -1       Error, the hash value cannot be calculated
 */
typedef int LLHFUNC(int key);

void llhinit(LLHTABLE t);
LLHITEM *llhlookup(LLHTABLE t, LLHFUNC *f, int key);
int llhinsert(LLHTABLE t, LLHFUNC *f, LLHITEM d);
int llhremove(LLHTABLE t, LLHFUNC *f, int key);
void llhclear(LLHTABLE t);
int llhsize(LLHTABLE t);

#endif
