/* Open.Net Authentication Daemon
 * Handles sending of statistics
 *
 * Written by Martin Hedenfalk <mhe@it.kth.se>
 *
 * Copyright (c) 2002, Royal Institute of Technology, Stockholm, Sweden
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/* $Id: stats.c,v 1.11 2002/04/19 11:51:15 mhe Exp $
 */

#include "common.h"

static req_status_t send_user_stats(FILE *fp, userdata_t *ud)
{
	fprintf(fp, "%s %s %s %s %s %lu\r\n",
			strmstatus(ud->mstatus), ud->username,
			ud->domain->name, inet_ntoa(ud->ip), strmac(ud->mac),
			ud->logintime.tv_sec);
	return REQ_STATUS_OK;
}

static req_status_t send_list_stats(FILE *fp, list_t *list)
{
	listitem_t *li;

	for(li = list->first; li; li = li->next) {
		req_status_t status;
		status = send_user_stats(fp, (userdata_t *)li->data);
		if(status != REQ_STATUS_OK)
			return status;
	}
	return REQ_STATUS_OK;
}

#if 0
req_status_t stats_send_domain(FILE *fp, char *domain)
{
	req_status_t status = REQ_STATUS_FAIL;
	int di;

	logmsg(LOG_INFO, "sending stats for domain %s", domain);

	di = find_domain_index(domain);
	if(di == -1)
		status = REQ_STATUS_NODOMAIN;
	else {
		pthread_mutex_lock(&modify_list_mutex);
		status = send_list_stats(fp, domains[di].users);
		pthread_mutex_unlock(&modify_list_mutex);
	}

	return status;
}
#endif

req_status_t stats_send_user(FILE *fp, char *username, char *domain)
{
	req_status_t status = REQ_STATUS_FAIL;
	userdata_t *ud = 0;

	logmsg(LOG_INFO, "sending stats for user %s@%s", username, domain);

	ud = search_user(username, domain);
	if(ud)
		status = send_user_stats(fp, ud);
	else
		status = REQ_STATUS_NOUSER;
	
	return status;
}

req_status_t stats_send(FILE *fp)
{
	req_status_t status;
	pthread_mutex_lock(&modify_list_mutex);
	status = send_list_stats(fp, users);
	pthread_mutex_unlock(&modify_list_mutex);
	return status;
}
